/*
 * #############################################################################
 * ATTENTION: BridgeMain must be started because of BridgeRMI (design fault) !?!
 * #############################################################################
 * 
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2004 by Walther Neuper 
 * created Aug 30, 2004, 11:58:35 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to license terms.
 */
package isac.bridge;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Vector;

import org.apache.log4j.Logger;

import isac.util.CalcChangedEvent;
import isac.util.Formalization;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcFormula;
import isac.util.interfaces.ICalcElement;
import isac.util.interfaces.ICalcIterator;
import isac.util.interfaces.IToCalc;
import isac.util.interfaces.IToUser;
import isac.util.tactics.Tactic;
import junit.framework.TestCase;

/**
 * @author Walther Neuper Aug 30, 2004, 11:58:35 AM
 */
public class TestBridge extends TestCase {

	static Logger logger_ = Logger.getLogger(MathEngine.class.getName());

	/**
	 * tests the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprising UC
	 * 'start auto calculation' \label{SPECIFY:START:auto}
	 * 
	 * see BridgeLog (sml <->java) in UC-mini-auto-step.txt
	 */
	public void testMinisubpblAutoCalc() throws Exception {
		System.out
				.println("---BEGIN isac.bridge.TestBridge#testMinisubpblAutoCalc");

		//prepare Formalization for start calculation in step_b
		Formalization fmzA = null, fmz = null;
		Object test = null;
		// the following would be instead reading from file ...
		//KEStoreServices ks = new
		//KEStoreServices("isac/src/java-tests/isac/kestore/data");
		//Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
		//String xmlstr = (String) vec.firstElement();
		//
		//FormalizationDigest fd = new FormalizationDigest();
		//fmz = fd.parseFormalization((String) xmlstr);
		// 
		File fAbroad = new File("isac/src/java-tests/isac/util/parser/data",
				"exp_IsacCore_Tests_1b.fmz");
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream oisA = new ObjectInputStream(new FileInputStream(
					fAbroad));
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmzA = (Formalization) oisA.readObject();
			fmz = (Formalization) ois.readObject();
			oisA.close();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		System.out.println(".FormalizationDigest.parseFormalization");
		//System.out.println("-fmz.toString()" + fmz.toString());
		assertEquals("mini-auto: fmz from 'abroad' file ", fmzA.toSMLString(),
				fmz.toSMLString());
		System.out.println(".end step_a-----");
		//end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//start calculation and complete CalcHead
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);
		// sent to bridge: CalcTree [(["equality (x+1=2)", "sol ...
		// sent to bridge: Iterator 1; #drop: make It.1 in CalcTree
		// sent to bridge: moveActiveRoot 1; #drop: make It.1 in CalcTree

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;

		ct.completeCalcHead();// ######informListeners NOW!!!
		// sent to bridge: autoCalculate 1 CompleteCalcHead;
		ICalcIterator ci = ct.getActiveFormula();
		//######^^^ drop: push addDataChangeListener towards/into
		//######^^^ startCalculation and fetch ci from CalcChanged:
		//ci = (ICalcIterator) cce.getLastGeneratedFormula().clone();
		CalcHead ch = (CalcHead) ci.getFormula();

//	too sensible against changes in CalcHead !!!	
//		f = new File("isac/src/java-tests/isac/bridge/data/mini-auto-step",
//				"step_b.calcHead");
//		//try {
//		//	ObjectOutputStream oos = new ObjectOutputStream(
//		//			new FileOutputStream(f));
//		//	oos.writeObject(ch);
//		//	oos.close();
//		//} catch (IOException e) {
//		//	e.printStackTrace();
//		//}
//		try {
//			ObjectInputStream ois = new ObjectInputStream(
//					new FileInputStream(f));
//			test = (CalcHead) ois.readObject();
//			ois.close();
//		} catch (IOException e) {
//			e.printStackTrace();
//		} catch (ClassNotFoundException e) {
//			e.printStackTrace();
//		}
//		System.out.println(".calchead: " + ch.toSMLString());
//		assertEquals("mini-auto: calcHead", ch.toSMLString(), ((CalcHead) test)
//				.toSMLString());
		System.out.println(".end step_b-----");
		//end step_b
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//autoCalculate
		System.out.println(".autoCalculate");
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);

		cce = ((MockIToUser) user).getCalcChangedEvent();
		assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
				"([],Met)");
		assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
				"([],Met)");
		assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
				"([],Res)");
		System.out.println(".end step_c-----");
		//end step_c
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//display the steps generated
		Object fch = null;
		ci = (ICalcIterator) cce.getLastUnchangedFormula().clone();
		int i = 0;
		while (ci.compareTo(cce.getLastGeneratedFormula()) < 0) {
			i++;
			boolean b = ci.moveDown();
			System.out.print("..iterator at " + ci.toSMLString());
			fch = ci.getFormula();
			System.out.println("   yields "
					+ ((ICalcElement) fch).toSMLString());
		}
//		Vector elems = ct.getElementsFromTo(cce.getLastUnchangedFormula(),
//				cce.getLastGeneratedFormula(), new Integer(2), false);
		System.out.println("mini-auto: steps= " + i);
		assertEquals("mini-auto: steps= ",i,10);
		assertEquals("mini-auto: last step",
				((ICalcElement) fch).toSMLString(), "[x = 1]");
		System.out.println(".end step_d-----");
		//end step_d
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		logger_.debug("---END isac.bridge.TestBridge#testMinisubpblAutoCalc");
		System.out
				.println("---END isac.bridge.TestBridge#testMinisubpblAutoCalc");
	}

	/***************************************************************************
	 * tests the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprising UC
	 * 'start interactive calculation' \label{SPECIFY:START:interactive} +
	 * \label{SOLVE:AUTO:one}}
	 * 
	 * step_a like above
	 */
	public void testMinisubpblNextStep() throws Exception {
		System.out
				.println("---BEGIN isac.bridge.TestBridge#testMinisubpblNextStep");
		Formalization fmzA = null, fmz = null;
		Object test = null;

		//\label{SPECIFY:START:interactive}
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmz = (Formalization) ois.readObject();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		System.out.println(".end step_a-----");
		//end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//BridgeMain _must_ run for the following ...
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();

		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;

		ct.completeCalcHead();
		ICalcIterator ci = ct.getActiveFormula();
		CalcHead ch = (CalcHead) ci.getFormula();

//		too sensible against changes in CalcHead !!!	
//		f = new File("isac/src/java-tests/isac/bridge/data/mini-auto-step",
//				"step_b.calcHead");
//		try {
//			ObjectInputStream ois = new ObjectInputStream(
//					new FileInputStream(f));
//			test = (CalcHead) ois.readObject();
//			ois.close();
//		} catch (IOException e) {
//			e.printStackTrace();
//		} catch (ClassNotFoundException e) {
//			e.printStackTrace();
//		}
//		System.out.println(".calchead: " + ch.toSMLString());
//		assertEquals("mini-step: calcHead", ch.toSMLString(), ((CalcHead) test)
//				.toSMLString());
		System.out.println(".end step_b-----");
		//end step_b
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//let calculate the first formula
		Tactic tac = ct.fetchProposedTactic();
		//sent to bridge: fetchProposedTactic 1;
		System.out.println(".2nd tac:" + tac.toSMLString());
		assertEquals("mini-step: 2nd form", tac.toSMLString(),
				"Apply_Method [\"Test\",\"squ-equ-test-subpbl1\"]");
		ct.setNextTactic(tac);
		//sent to bridge: setNextTactic 1
		// (Apply_Method["Test","squ-equ-test-subpbl1"]);
		//sent to bridge: autoCalculate 5 (Step 1);
		//^^^implicit in setNextTactit; remove ^^^; see @@@
		//###ct.autoCalculate(IToCalc.SCOPE_CALCULATION,1);//decomment ########

		cce = ((MockIToUser) user).getCalcChangedEvent();
		assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
				"([1],Frm)");//"([],Pbl)");...correct, but see
		// CalcTree#setNextTactic
		assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
				"([1],Frm)");//"([],Pbl)");...correct, but see
		// CalcTree#setNextTactic
		assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
				"([1],Frm)");

		CalcFormula fa = (CalcFormula) (cce.getLastGeneratedFormula()).getFormula();
		System.out.println(".2nd form: " + fa.toSMLString());
		assertEquals("mini-auto: 2nd form", fa.toSMLString(), "x + 1 = 2");

		System.out.println(".end step_d-----");
		//end step_c
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//let calculate the second formula
		tac = ct.fetchProposedTactic();
		System.out.println(".3rd tac: " + tac.toSMLString());
		assertEquals("mini-step: 3rd tac", tac.toSMLString(),
				"Rewrite_Set \"norm_equation\"");

		ct.setNextTactic(tac);
		//sent to bridge: autoCalculate 5 (Step 1);
		//^^^implicit in setNextTactit; remove ^^^; see @@@
		//##ct.autoCalculate(IToCalc.SCOPE_CALCULATION,1);//decomment ##
		cce = ((MockIToUser) user).getCalcChangedEvent();
		//@@@@@error@@@error@@@error@@@error@@@error@@@----"([1],Res)"
		assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
				"([1],Res)");//"([1],Frm)");...correct, but see
		// CalcTree#setNextTactic
		//@@@@@error@@@error@@@error@@@error@@@error@@@----"([1],Res)"
		assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
				"([1],Res)");//"([1],Frm)");...correct, but see
		// CalcTree#setNextTactic
		assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
				"([1],Res)");

		fa = (CalcFormula) (cce.getLastGeneratedFormula()).getFormula();
		System.out.println(".3rd form: " + fa.toSMLString());
		assertEquals("mini-auto: 3rd form", fa.toSMLString(),
				"x + 1 + -1 * 2 = 0");

		System.out.println(".end step_e-----");
		//end step_d
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//let calculate the third formula
		tac = ct.fetchProposedTactic();
		System.out.println(".4th tac: " + tac.toSMLString());// <>Test_simplify???
		assertEquals("mini-step: 4th tac", tac.toSMLString(),
				"Rewrite_Set \"Test_simplify\"");

		ct.setNextTactic(tac);
		cce = ((MockIToUser) user).getCalcChangedEvent();
		//@@@@@error@@@error@@@error@@@error@@@error@@@----"([2],Res)"
		assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
				"([2],Res)");//"([1],Res)");...correct, but see
		// CalcTree#setNextTactic
		//@@@@@error@@@error@@@error@@@error@@@error@@@----"([2],Res)"
		assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
				"([2],Res)");//"([1],Res)");...correct, but see
		// CalcTree#setNextTactic
		assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
				"([2],Res)");

		fa = (CalcFormula) (cce.getLastGeneratedFormula()).getFormula();
		System.out.println(".4th form:" + fa.toSMLString());
		assertEquals("mini-auto: 4th form", fa.toSMLString(), "-1 + x = 0");

		System.out.println(".end step_f-----");
		//end step_e
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	}

	/**
	 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
	 * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
	 * sml/systest/FE-interface.sml "...enter} right"
	 */
	public void testAppendRight() throws Exception {
		System.out.println("---BEGIN isac.bridge.TestBridge#testAppendRight");
		// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
		//prepare Formalization for start calculation in step_b
		Formalization fmz = null;
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmz = (Formalization) ois.readObject();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		//start calculation
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;
		//do the first two steps
		ct.completeCalcHead();
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
		//		ICalcIterator ci = ct.getActiveFormula();
		//		Formula fa = (Formula) ci.getFormula();
		//		assertEquals("x + 1 + -1 * 2 = 0",fa.toSMLString());
		//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//append "-1 + x = 0" which the kernel would have proposed, too
		CalcFormula in = new CalcFormula();
		in.setText("-1 + x = 0");
		ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
		cce = ((MockIToUser) user).getCalcChangedEvent();
		assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
				"([1],Res)");
		assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
				"([1],Res)");
		assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
				"([2],Res)");

		//no formulae have been inserted:
		cce.getLastUnchangedFormula().moveDown();
		assertEquals(cce.getLastUnchangedFormula().toSMLString(), cce
				.getLastGeneratedFormula().toSMLString());
		logger_.debug("---END isac.bridge.TestBridge#testAppendRight");
	}

	/**
	 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
	 * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
	 * sml/systest/FE-interface.sml "...enter} other"
	 */
	public void testAppendOther1() throws Exception {
		System.out.println("---BEGIN isac.bridge.TestBridge#testAppendOther1");
		// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
		//prepare Formalization for start calculation in step_b
		Formalization fmz = null;
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmz = (Formalization) ois.readObject();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		//start calculation
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;
		//do the first two steps
		ct.completeCalcHead();
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
		//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//append "x - 1 = 0" which needs 11 steps (on same level) to justify
		CalcFormula in = new CalcFormula();
		in.setText("-1 + x = 0");
		ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
		cce = ((MockIToUser) user).getCalcChangedEvent();
		assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
				"([1],Res)");
		assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
				"([1],Res)");
		assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
				"([2],Res)");

		//the inserted formulae need _NOT_ to be displayed:
		ICalcIterator ci = (ICalcIterator) cce.getLastUnchangedFormula()
				.clone();
		cce.getLastUnchangedFormula().moveDown();//!!!!!!!!!!
		assertEquals(cce.getLastUnchangedFormula().toSMLString(), cce
				.getLastGeneratedFormula().toSMLString());
		assertEquals(cce.getLastGeneratedFormula().toSMLString(), ct
				.getActiveFormula().toSMLString());

		//but there are 11 steps in between
		for (int i = 1; i <= 11; i++)
			ci.moveDown();
		assertEquals(ci.toSMLString(), cce.getLastGeneratedFormula()
				.toSMLString());
		logger_.debug("---END isac.bridge.TestBridge#testAppendOther1");
	}

	/**
	 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
	 * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
	 * sml/systest/FE-interface.sml "...enter} oth 2"
	 */
	public void testAppendOther2() throws Exception {
		System.out.println("---BEGIN isac.bridge.TestBridge#testAppendOther2");
		// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
		//prepare Formalization for start calculation in step_b
		Formalization fmz = null;
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmz = (Formalization) ois.readObject();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		//start calculation
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;
		//do the first two steps
		ct.completeCalcHead();
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
		//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//append "x = 1" which needs 6 steps (+ one level down) to justify
		CalcFormula in = new CalcFormula();
		in.setText("x = 1");
		ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
		cce = ((MockIToUser) user).getCalcChangedEvent();
		assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
				"([1],Res)");
		assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
				"([1],Res)");
		assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
				"([3,2],Res)");

		//the inserted formulae _MUST_ be displayed (but not on level 3 !):
		ICalcIterator ci = (ICalcIterator) cce.getLastUnchangedFormula()
				.clone();
		for (int i = 1; i <= 8; i++) {
					ci.moveDown();
				    System.out.println(".step "+i+": "+ci.toSMLString() + " "
		+ ci.getFormula().toSMLString());
				}
		assertEquals(ci.toSMLString(), cce.getLastGeneratedFormula()
				.toSMLString());
		assertEquals(cce.getLastGeneratedFormula().toSMLString(), ct
				.getActiveFormula().toSMLString());

		logger_.debug("---END isac.bridge.TestBridge#testAppendOther2");
	}

	/**
	 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
	 * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
	 * sml/systest/FE-interface.sml "...enter} NOTok"
	 */
	public void testAppendNotOk() throws Exception {
		System.out.println("---BEGIN isac.bridge.TestBridge#testAppendNotOk");
		// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
		//prepare Formalization for start calculation in step_b
		Formalization fmz = null;
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmz = (Formalization) ois.readObject();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		//start calculation
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;
		//do the first two steps
		ct.completeCalcHead();
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
		//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

		//append "x = 1" which needs 6 steps (+ one level down) to justify
		CalcFormula in = new CalcFormula();
		in.setText("x = 4711");
		ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
		//zum Zurueckmelden von <ERROR> no derivation found </ERROR>
		//brauchen wir schon den neuen CalcChangedEvent !!!

		logger_.debug("---END isac.bridge.TestBridge#testAppendNotOk");
	}

	/**
	 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
	 * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
	 * sml/systest/FE-interface.sml "...replace} right"
	 */
	public void testReplaceRight() throws Exception {
		System.out.println("---BEGIN isac.bridge.TestBridge#testReplaceRight");
		// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
		//prepare Formalization for start calculation in step_b
		Formalization fmz = null;
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmz = (Formalization) ois.readObject();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		//start and finish the calculation
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
		cce = ((MockIToUser) user).getCalcChangedEvent();
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		//TODO
		logger_.debug("---END isac.bridge.TestBridge#testReplaceRight");
	}

	/**
	 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
	 * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
	 * sml/systest/FE-interface.sml "...replace} other"
	 */
	public void testReplaceOther1() throws Exception {
		System.out.println("---BEGIN isac.bridge.TestBridge#testReplaceOther1");
		// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
		//prepare Formalization for start calculation in step_b
		Formalization fmz = null;
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmz = (Formalization) ois.readObject();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		//start calculation
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;
		//autoCalculate
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
		cce = ((MockIToUser) user).getCalcChangedEvent();
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		//TODO
		logger_.debug("---END isac.bridge.TestBridge#testReplaceOther1");
	}

	/**
	 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
	 * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
	 * sml/systest/FE-interface.sml "...replace} other 2"
	 */
	public void testReplaceOther2() throws Exception {
		System.out.println("---BEGIN isac.bridge.TestBridge#testReplaceOther2");
		// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
		//prepare Formalization for start calculation in step_b
		Formalization fmz = null;
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmz = (Formalization) ois.readObject();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		//start calculation
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;
		//autoCalculate
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
		cce = ((MockIToUser) user).getCalcChangedEvent();
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		//TODO
		logger_.debug("---END isac.bridge.TestBridge#testReplaceOther2");
	}

	/**
	 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
	 * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
	 * sml/systest/FE-interface.sml "...replace} NOTok"
	 */
	public void testReplaceNotOk() throws Exception {
		System.out.println("---BEGIN isac.bridge.TestBridge#testReplaceNotOk");
		// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
		//prepare Formalization for start calculation in step_b
		Formalization fmz = null;
		File f = new File(
				"isac/src/java-tests/isac/bridge/data/mini-auto-step",
				"step_a.fmz");
		try {
			ObjectInputStream ois = new ObjectInputStream(
					new FileInputStream(f));
			fmz = (Formalization) ois.readObject();
			ois.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		//start calculation
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;
		//autoCalculate
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
		cce = ((MockIToUser) user).getCalcChangedEvent();
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		//TODO
		logger_.debug("---END isac.bridge.TestBridge#testReplaceNotOk");
	}

}