/*
 * Created on Aug 13, 2003
 */
package isac.bridge;

/**
 * @author rgradisc
 */

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * This class runs in its own thread,
 * fetches the output from the kernel, and
 * sends it back to the clients
 * @author richard
 */
public class Kernel2ClientsServer implements Runnable {
  //Start and end sequences to mark XML parts in SML output
  private static final String START_SEQ = "@@@@@begin@@@@@";
  private static final String END_SEQ = "@@@@@end@@@@@";
  
  private ClientList clientList;
  private BridgeMain bridge;
  private Vector timeOutTimes;

  static Logger logger = Logger.getLogger(Kernel2ClientsServer.class.getName());
  
  //---------------------------------------------------------------
  public Kernel2ClientsServer(BridgeMain bridge) {
  	logger.warn("Kernel2ClientsServer: bridge="+bridge);
    //this.smlReader = bridge.getSmlReader();
    this.clientList = bridge.getClientList();
    this.timeOutTimes = bridge.getTimeOutTimes();
    this.bridge = bridge;
  }

  //---------------------------------------------------------------
  public void run() {
    String line;
    boolean startedXML = false;
    int indexStart;
    int indexEnd;
    int currID = 0;
    StringBuffer sb = new StringBuffer();
    while (true) {
      try {
        line = bridge.getSmlReader().readLine();
        if (bridge.ignoreOutput == 0) {
          bridge.log(1, "[[from sml: " + line);
        }
        if (startedXML) {
          indexEnd = line.indexOf(END_SEQ);
          if (indexEnd != -1) {
            sb.append(line.substring(0, indexEnd));
            if (timeOutTimes.size() > 0) {
              timeOutTimes.removeElementAt(0);
            }
            // remove first (oldest) element from timeOutTimes

            if (bridge.ignoreOutput == 0) {
              // send XML-string (response of sml-kernel) to client
              String s = new String(sb);
              //TODO: FIXXME: removes the id in response:
              s = s.substring(s.indexOf("<"));
              //TODO: FIXXME: only one client
              //currID = XMLParser.parseCalcID(s);
              //TODO fetch id from bridge
              //so far: only one bridgeRMI
              currID = 1;
              bridge.log(1, "found CalcID: " + currID);
              PrintWriter pw = clientList.getPrintWriterOfClient(currID);
              if (pw != null) {
                pw.println(s);
                bridge.log(1, ">> Response sent to user " + pw.toString());
              }
              //bridge.responseSent();             
            } else {
              bridge.ignoreOutput--;
              bridge.log(1, "ignored output (restore kernel state)");
            }
            startedXML = false;
          } else {
            sb.append(line);
          }
        } else {
          indexStart = line.indexOf(START_SEQ);
          if (indexStart != -1) {
            sb = new StringBuffer();
            startedXML = true;
          }
        }       
      } catch (IOException e) {
        //bridge.log(1, "Read from sml-kernel failed, waiting for kernel to restart");                
      } catch (NullPointerException e) {
        bridge.log(1, "Read from sml failed, waiting for restart...");
      } catch (IndexOutOfBoundsException e) {
        bridge.log(1, "Kernel2Clients: Index out of bound");
        e.printStackTrace();
      } catch (NumberFormatException e) {
        bridge.log(1, "Couldn't parse user ID: NumberformatException");        
      }
    }
  }
}
