/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 					
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui;

import isac.gui.calcheadviews.CalcHeadView;
import isac.gui.calcheadviews.FullCalcHeadView;
import isac.gui.calcheadviews.SimpleCalcHeadView;
import isac.util.formulae.CalcHead;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Locale;

import javax.swing.JPanel;

import org.apache.log4j.Logger;

/**
 * Panel where the user can input/see a complete CalcHead with all its
 * Elements.
 * 
 * @author Mario Hochreiter
 * @version 0.1
 *
 */
public class CalcHeadPanel extends JPanel implements ActionListener {
	static Logger logger = Logger.getLogger(CalcHeadPanel.class.getName());  
  
  public static final int TYPE_TRY_MATCH = 0;
  public static final int TYPE_TRY_REFINE = 1;
    
  /** changeable view to the data of the calchead */
  protected CalcHeadView infoPanel;
  protected CalcHead calcHead;

  /**
   * Createas a <code>CalcHeadPanel</code> with the given <code>CalcHead</code>.
   * The displayed detail level of the <code>CalcHead</code> is set through
   * viewStyle.
   *  
   * @param locale Locale that is used to display the CalcHeadPanel strings.
   * @param ch CalcHead that should be displayed.
   * @param viewStyle The requested level of detail for the display of the
   * CalcHead 
   */
  public CalcHeadPanel(Locale locale, 
                       CalcHead ch, int viewStyle) {
    this.init(locale, ch, viewStyle);    
  }
  
  protected void init(Locale locale, CalcHead ch, int viewStyle) {
    logger.debug("viewStyle: " + viewStyle);
    this.calcHead = ch;
    if (viewStyle == CalcHead.VIEWSTYLE_FULL)
      infoPanel = new FullCalcHeadView(locale, ch);
    else if (viewStyle == CalcHead.VIEWSTYLE_SIMPLE_SOLVE) {
      logger.debug("simpleViewStyle");
      infoPanel = new SimpleCalcHeadView(locale, ch);
    } else //default ViewStyle
      infoPanel = new FullCalcHeadView(locale, ch);
    this.layoutComponents();
  } 
  
  protected void layoutComponents() {
   this.setLayout(new BorderLayout());
   this.add(infoPanel, BorderLayout.CENTER);
  }

  public void fillTextFieldsFromCalcHead(CalcHead ch) {
    infoPanel.fillTextFieldsFromCalcHead(ch);
  }
  
  public void fillCalcHeadFromTextFields(CalcHead ch) {
    infoPanel.fillCalcHeadFromTextFields(ch);
  }
  
  public void setCalcHead(CalcHead ch) {
    infoPanel.fillTextFieldsFromCalcHead(ch);
    // calculate the new sizes of info Panel, as the text has changed
    //infoPanel.validate();
    this.validate();
  }

  //-------------------ActionListener---------------------------------------------
  public void actionPerformed(ActionEvent e) {
   
  }
}
