/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.browser;

import isac.gui.browser.example.ExampleBrowserPanel;
import isac.gui.browser.method.MethodBrowserPanel;
import isac.gui.browser.problem.ProblemBrowserPanel;
import isac.gui.util.Hierarchy;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

/**
 * Panel in the window application that is responsible to layout the
 * browser windows (e.g. example browser). The different browser windows
 * are shown as different tabs in a JTabbedPane. 
 * 
 * @author Mario Hochreiter
 * @version 0.1
 *
 */
public class BrowserPanel extends JPanel {
  /** Contains all "browser" panels like "Example browser". */
  private JTabbedPane browserTabPane;
  
  /** The mother frame */
  private JFrame root;  
    
  public BrowserPanel(JFrame root) {
    this.root = root;
    this.setMinimumSize(new Dimension(200, 200));
    browserTabPane = new JTabbedPane(JTabbedPane.BOTTOM, 
                                     JTabbedPane.SCROLL_TAB_LAYOUT);
    this.setLayout(new BorderLayout());
    this.add(browserTabPane, BorderLayout.CENTER);
  } 
  
  public void initExamplesBrowser(Hierarchy hierarchy) {
    browserTabPane.addTab("Example Browser", new ExampleBrowserPanel(root, hierarchy)); 
  }
  
  public void initProblemBrowser(Hierarchy hierarchy) {
    browserTabPane.addTab("Problem Browser", new ProblemBrowserPanel(root, hierarchy));
  }
  
  public void initMethodBrowser(Hierarchy hierarchy) {
    browserTabPane.addTab("Method Browser", new MethodBrowserPanel(root, hierarchy));
    browserTabPane.addTab("Theory Browser", new JPanel());    
  }
}
