/*
 * Created on Aug 16, 2003
 */
package isac.util;

import isac.browserdialog.BDialog;
import isac.session.OManager;
import isac.session.SDialog;

import java.io.IOException;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.Vector;

import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;

/**
 * @author Andreas Griesmayer
 * 
 * The InformationProcessor builds the bridge between BrowserFrontend and
 * BrowserDialog. It encapsulates all tasks regarding communication and connects
 * to the proper dialog-modules.
 * 
 * Because the final communication-protocol is not fixed yet, it provides
 * methods to communicate via RMI as well as XML-RPC
 */
public class InformationProcessor {

	/**
	 * Performes an login on the session dialog using the XML-RPC protocol. It
	 * assumes that the SessionDialog is running its XML-RPC-handler on port
	 * 1050 of the local machine.
	 * 
	 * @param username
	 *            username of the user who wants to login.
	 * @param password
	 *            corresponding password for the user.
	 * @return the id of the current sesson the user logged in or null if the
	 *         login did not succeed
	 * @throws NullPointerException
	 *             carrying the error-message if an communication error occured.
	 */
	static protected String xmlLogin(String username, String password) {
		try {
			XmlRpcClient sdialog = new XmlRpcClient("http://localhost:1050");
			Vector params = new Vector();
			params.add(username);
			params.add(password);
			Vector response = (Vector) sdialog.execute("SDialog.login", params);
			if (response.size() > 0)
				return (String) response.elementAt(0);
		} catch (MalformedURLException e) {
			throw new NullPointerException(
					getErrorXML("Sorry, MalformedURLException while accessing the SDialog to login - is the SDialog runnig ?"
							+ e.getMessage()));
		} catch (XmlRpcException e) {
			throw new NullPointerException(
					getErrorXML("Sorry, XmlRpcException while accessing the SDialog to login - is the SDialog runnig ?"
							+ e.getMessage()));
		} catch (IOException e) {
			throw new NullPointerException(
					getErrorXML("Sorry, IOException while accessing the SDialog to login - is the SDialog runnig ?"
							+ e.getMessage()));
		}
		return null;
	}

	/**
	 * Performes a login on the Session dialog using RMI. It assumes the
	 * SessionDialog to be found at //localhost/isac-ObjectManager
	 * 
	 * @param username
	 *            username of the user who wants to login.
	 * @param password
	 *            corresponding password for the user.
	 * @return the id of the current sesson the user logged in or null if the
	 *         login did not succeed
	 * @throws NullPointerException
	 *             carrying the error-message if an communication error occured.
	 */
	static public String login(String username, String password) {
		SDialog sd = getSDialog();
		try {
			/* LK 04.08.27 */
			//if username != pwd -> empty Vector
			Vector v = sd.login(username, password);
			if (v.size() == 0) {
				return null;
			} else {
				return (String) v.elementAt(0);
			}
		} catch (RemoteException e) {
			throw new NullPointerException(
					getErrorXML("Sorry, RemoteEcxeption while accessing the SDialog to login - is the SDialog runnig ?"
							+ e.getMessage()));
		}
	}

	public static BDialog getBDialog(String session) {
		String name = "//localhost/isac-ObjectManager";
		try {
			System.out.println("connect to ObjectManager" + name);
			Object lookup = Naming.lookup(name);
			return ((OManager) lookup).getBrowserDialog(session);
		} catch (RemoteException exc) {
			System.err.println("no rmiregistry runing on server " + name
					+ " no connection possible --- try later");
			System.err.println("serverCom exception: " + exc.getMessage());
		} catch (NotBoundException exc) {
			System.err.println("servercom: notboundexc: " + exc.getMessage());
			System.err.println("server not found - try to connect to Proxy");
		} catch (java.net.MalformedURLException exc) {
			System.err.println("servercom: MalformedURLException: "
					+ exc.getMessage());
			exc.printStackTrace();
		}
		return null;
	}

	public static SDialog getSDialog() {
		String name = "//localhost/isac-ObjectManager";
		try {
			System.out.println("connect to ObjectManager: " + name);
			Object lookup = Naming.lookup(name);
			return ((OManager) lookup).getSessionDialog();
		} catch (RemoteException exc) {
			System.err.println("no rmiregistry runing on server " + name
					+ " no connection possible --- try later");
			System.err.println("serverCom exception: " + exc.getMessage());
		} catch (NotBoundException exc) {
			System.err.println("servercom: notboundexc: " + exc.getMessage());
			System.err.println("server not found - try to connect to Proxy");
		} catch (java.net.MalformedURLException exc) {
			System.err.println("servercom: MalformedURLException: "
					+ exc.getMessage());
			exc.printStackTrace();
		}
		return null;
	}

	/*
	 * static protected String oldloadString(String function, Vector parameter){
	 * XmlRpcClient bdialog; String xmlsource=""; try { bdialog = new
	 * XmlRpcClient("http://localhost:1040"); Vector result = (Vector)
	 * bdialog.execute(function, parameter); xmlsource =
	 * (String)result.firstElement(); } catch (MalformedURLException e) {
	 * e.printStackTrace(); } catch (XmlRpcException e) { e.printStackTrace(); }
	 * catch (IOException e) { e.printStackTrace(); } return xmlsource; }
	 */
	/*
	 * static protected Vector oldloadVector(String function, Vector parameter){
	 * Vector return_vect=new Vector(); XmlRpcClient bdialog; String
	 * xmlsource=""; try { bdialog = new XmlRpcClient("http://localhost:1040");
	 * return_vect = (Vector) bdialog.execute(function, parameter); } catch
	 * (MalformedURLException e) { e.printStackTrace(); } catch (XmlRpcException
	 * e) { e.printStackTrace(); } catch (IOException e) { e.printStackTrace(); }
	 * return return_vect; }
	 */

	/**
	 * Loads the hierarchy for the information-type <code>type</type>
	 * @param session session to identify on the SessionDialog
	 * @param type type of the hierarchy to load
	 * @return  the requested hierarchy in its XML-representation, in case of an RemoteException, a XML-code 
	 * carrying the error-message is returned
	 */
	static public String loadHierarchy(String session, String type) {
		BDialog bd = getBDialog(session);
		try {
			return bd.loadHierarchy(type);
		} catch (RemoteException e) {
			return getErrorXML("Sorry, RemoteEcxeption while accessing the BDialog to login - is the BDialog runnig ?"
					+ e.getMessage());
		}
	}

	/**
	 * Loads binary files from the KE-Base
	 * 
	 * @param session
	 *            session to identify on the SessionDialog
	 * @param type
	 *            to identify the location of the binary file. An id has only to
	 *            be unique within a type, although a globally unique id should
	 *            be used.
	 * @param content_id
	 *            Id of the binary-file to load
	 * @return the requested binary-file as byte-array, in case of an
	 *         RemoteException, an empty array is returned
	 */
	static public byte[] loadBinary(String session, String type,
			String content_id) {
		BDialog bd = getBDialog(session);
		try {
			return bd.loadBinary(type, content_id);
		} catch (RemoteException e) {
			return new byte[0];
		}
	}

	static public String loadXML(String session, String type, Vector idvect) {
		BDialog bd = getBDialog(session);
		try {
			return bd.loadContent(type, idvect);
		} catch (RemoteException e) {
			return getErrorXML("Sorry, RemoteException while accessing the BDialog to fetch your content");
		}
	}

	/**
	 * Loads content-files from the KE-Base
	 * 
	 * @param session
	 *            session to identify on the SessionDialog
	 * @param type
	 *            to identify the location of the KE-Object. An id has only to
	 *            be unique within a type, although a globally unique id should
	 *            be used.
	 * @param content_id
	 *            Id of the KE-Object to load
	 * @return the requested KE-Object in its XML-representation, in case of an
	 *         RemoteException, a XML-code carrying the error-message is
	 *         returned
	 */
	static public String loadXML(String session, String type, String content_id) {
		BDialog bd = getBDialog(session);
		try {
			return bd.loadContent(type, content_id);
		} catch (RemoteException e) {
			return getErrorXML("Sorry, RemoteException while accessing the BDialog to fetch your file \""
					+ content_id + "\" is the BDialog running?");
		}
	}

	/**
	 * Evaluates the requested content in the given worksheet using the
	 * BrowserDialog
	 * 
	 * @param session
	 *            session to identify on the SessionDialog
	 * @param type
	 *            to identify the location of the KE-Object. An id has only to
	 *            be unique within a type, although a globally unique id should
	 *            be used.
	 * @param content_id
	 *            Id of the KE-Object to load
	 * @param wd_id
	 *            id of the worksheet the contanct for evaluating
	 * @return the requested KE-Object in its XML-representation, in case of an
	 *         RemoteException, a XML-code carrying the error-message is
	 *         returned
	 * @see BDialog#evaluateContent
	 */
	static public String evaluateXML(String session, String type,
			String content_id, String wd_id) {
		BDialog bd = getBDialog(session);
		try {
			return bd.evaluateContent(type, content_id, wd_id);
		} catch (RemoteException e) {
			return getErrorXML("Sorry, RemoteException while accessing the BDialog to evaluate \""
					+ content_id + "\" is the BDialog running?");
		}
	}

	/*
	 * static protected Reader loadsKEObject(String function, Vector parameter){
	 * XmlRpcClient bdialog; String xmlsource=""; try { bdialog = new
	 * XmlRpcClient("http://localhost:1040"); Vector result = (Vector)
	 * bdialog.execute(function, parameter); if(result.size()>0) xmlsource =
	 * (String)result.firstElement(); else xmlsource = ""; } catch
	 * (MalformedURLException e) { return new StringReader(getErrorXML("Sorry,
	 * MalformedURLException while accessing the KEStore")); } catch
	 * (XmlRpcException e) { return new StringReader(getErrorXML("Sorry,
	 * XmlRpcException while accessing the KEStore")); } catch (IOException e) {
	 * return new StringReader(getErrorXML("Sorry, IOException while accessing
	 * the KEStore")); } return new StringReader(xmlsource); }
	 */

	/**
	 * Get the corresponding user for a session.
	 * 
	 * @param session_id
	 *            id of the session to identify the user
	 * @return username or null if an invalid sessionid was given
	 * @throws NullPointerException
	 *             carrying the error-message if an RemoteException occures.
	 */
	public static String getUsername(String session_id) {
		SDialog sd = getSDialog();
		try {
			String username;
			if ((username = sd.getUsername(session_id)).equals(""))
				return null;
			return username;
		} catch (RemoteException e) {
			throw new NullPointerException(
					getErrorXML("Sorry, RemoteException while accesing the SDialog - is it running ?"));
		}
	}

	/**
	 * @param session_id
	 * @return
	 */
	private static String xmlGetUsername(String session_id) {
		XmlRpcClient bdialog;
		String xmlsource = "";
		Vector parameter = new Vector();
		parameter.add(session_id);
		try {
			bdialog = new XmlRpcClient("http://localhost:1050");
			String result = (String) bdialog.execute("SDialog.getUsername",
					parameter);
			if (result.equals(""))
				return null;
			return result;
		} catch (MalformedURLException e) {
			return getErrorXML("Sorry, MalformedURLException while accesing the SDialog - is it running ?");
		} catch (XmlRpcException e) {
			return getErrorXML("Sorry, XmlRpcException while accesing the SDialog - is it running ?");
		} catch (IOException e) {
			return getErrorXML("Sorry, IOException while accesing the SDialog - is it running ?");
		}
	}

	private static String getErrorXML(String string) {
		return "<ERROR><DESCRIPTION> <h1> Error in file InformationPrecessor.java </h1>"
				+ string + "</DESCRIPTION></ERROR>";
	}
}