/*
 * Created on Sep 11, 2003
 */
package isac.util.formulae;

/**
 * Every calculation has a CalcHead, which consists of Model and Specification
 * of the calculation The CalcHead is empty at the beginning of the
 * Specifying-phase and has to be filled before one can start the solving phase.
 * 
 * @author Alan Krempler
 */
public class CalcHead extends CalcElement {

	/**
	 * Labels a model item as "correct" TODO rename CALCHEAD_* to MODEL_ITEM_*
	 */
	public static final int CALCHEAD_CORRECT = 1;

	/**
	 * Labels a model item as "incorrect"
	 */
	public static final int CALCHEAD_INCORRECT = 2;

	/**
	 * Labels a model item as "syntax error"
	 */
	public static final int CALCHEAD_SYNTAXERROR = 3;

	/**
	 * Labels a model item as "incomplete"
	 */
	public static final int CALCHEAD_INCOMPLETE = 4;

	/**
	 * Labels a model item as "false"
	 */
	public static final int CALCHEAD_FALSE = 5;

	/**
	 * Labels a model item as to be filled in by the KI
	 */
	public static final int CALCHEAD_HELP_ME = 6;

	/**
	 * Labels a model item as superfluous
	 */
	public static final int CALCHEAD_SUPERFLUOUS = 7;

	/**
	 * Labels a model item as missing
	 */
	public static final int CALCHEAD_MISSING = 8;

	public static final int VIEWSTYLE_FULL = 2;

	public static final int VIEWSTYLE_SINGLELINE = 4;

	public static final int VIEWSTYLE_IN_CALC = 5;

	public static final int VIEWSTYLE_SIMPLE_SOLVE = 10;

	// TODO: remove testing constant
	private int calcTreeID = -4711;

	private CalcHeadItemList given_, where_, find_, relate_;

	private Specification specification;

	private Position position;

	private String belongsTo;

	private String head;

	private int calcHeadStatus;//TODO.WN041007 confusion with statusString

	private String statusString;//TODO.WN041007 confusion with calcHeadStatus

	// drop THIS one and setStatusString

	/**
	 * Format the object in a way that the bridge can handle it
	 * FIXME.WN041008 this is Formalization.toSMLString, _NOT_
	 * CalcHead.toSMLString !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 * 
	 * @return SML representation
	 */
	public String toSMLString() {
		//this should go to Formalization
		StringBuffer sb = new StringBuffer();
		sb.append("[([");
		if (getGiven() != null) {
			sb.append(getGiven().toSMLString());
		}
		if (getWhere() != null) {
			sb.append(",");
			sb.append(getWhere().toSMLString());
		}
		if (getFind() != null) {
			sb.append(",");
			sb.append(getFind().toSMLString());
		}
		if (getRelate() != null) {
			sb.append(",");
			sb.append(getRelate().toSMLString());
		}
		sb.append("],(");
		if (getSpecification() != null) {
			sb.append(specification.toSMLString());
		}
		sb.append("))]");
		return sb.toString();
	}

	/**
	 * Special string format of the calcHead used by modifyCalcHead
	 * TODO.WN041007 ?
	 * 
	 * @return String used by modifyCalcHead
	 */
	public String toModifyCalcHeadString() {
		StringBuffer sb = new StringBuffer();
		try {
			belongsTo = "Pbl";//TODO.WN041007 this is too restrictive: + Met !!
			sb.append("((");
			if (position == null) {
				position = new Position();
				position.setKind("Pbl");
			}
			sb.append(position.toSMLString());
			sb.append("),\"");
			sb.append(head);
			sb.append("\",[Given [");
			sb.append(given_.toSMLString());
			sb.append("],Find [");
			sb.append(find_.toSMLString());
			sb.append("],Relate [");
			sb.append(relate_.toSMLString());
			sb.append("]]," + belongsTo + ",(");
			sb.append(specification.toSMLString());
			sb.append("))");

		} catch (Exception e) {
			System.out.println("Some Items of Calchead are null!");
		}
		return sb.toString();
	}

	/**
	 * Copies values form one CalcHead object to another. This is needed,
	 * because the backend produces a new calcHead after each update, but the
	 * dialog holds only one instance of CalcHead all the time.
	 * 
	 * @param other
	 *            the other calcHead object
	 */
	public void fillValuesfrom(CalcHead other) {
		this.belongsTo = other.belongsTo;
		this.calcHeadStatus = other.calcHeadStatus;
		this.calcTreeID = other.calcTreeID;
		this.find_ = other.find_;
		this.given_ = other.given_;
		this.head = other.head;
		this.position = other.position;
		this.relate_ = other.relate_;
		this.specification = other.specification;
		this.where_ = other.where_;
		this.visibility_ = other.visibility_;
		this.view_style_ = other.view_style_;
		this.rating_ = other.rating_;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see isac.util.CalcElement#getMathML()
	 */
	public String getMathML() {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see isac.util.CalcElement#getType()
	 */
	public int getType() {
		return CALCEL_CALCHEAD;
	}

	public CalcHeadItemList getFind() {
		return find_;
	}

	public CalcHeadItemList getGiven() {
		return given_;
	}

	public CalcHeadItemList getRelate() {
		return relate_;
	}

	public CalcHeadItemList getWhere() {
		return where_;
	}

	public void setFind(CalcHeadItemList el) {
		find_ = el;
	}

	public void setGiven(CalcHeadItemList el) {
		given_ = el;
	}

	public void setRelate(CalcHeadItemList el) {
		relate_ = el;
	}

	public void setWhere(CalcHeadItemList el) {
		where_ = el;
	}

	public Specification getSpecification() {
		return specification;
	}

	public void setSpecification(Specification specification) {
		this.specification = specification;
	}

	public Position getPosition() {
		return position;
	}

	public void setPosition(Position position) {
		this.position = position;
	}
	//for XMLParserDigest
	public void setResponse(Position position) {
		this.position = position;
	}

	public String getBelongsTo() {
		return belongsTo;
	}

	public void setBelongsTo(String string) {
		belongsTo = string;
	}

	public String getHead() {
		return head;
	}

	/**
	 * set the headline in the CalcHead; this is in general e.g.
	 * 'Problem(Reals,[by_calculus,maximum])' or for algebrasystem-like problems
	 * e.g. 'solve(x+1=2,x)'
	 */
	public void setHead(String string) {
		head = string;
	}

	public int getCalcTreeID() {
		return calcTreeID;
	}

	public void setCalcTreeID(int id) {
		calcTreeID = id;
	}

	public int getCalcHeadStatus() {
		return calcHeadStatus;
	}

	/**
	 * distinguish between (1) the _overall_-status of a CalcHead and (2) e.g.
	 * the CALCHEAD_CORRECT for model items as defined above HERE (2) IS MEANT !
	 * 
	 * @see <isac.util.formulae.CalcHead#setStatusString(String)>
	 */
	public void setCalcHeadStatus(int i) {
		calcHeadStatus = i;
	}

	public void setCalcHeadStatus(String s) {//TOD0.WN041007 drop this NONSENSE
		if (s.compareTo("correct") == 0) {
			calcHeadStatus = (CALCHEAD_CORRECT);
		} else if (s.compareTo("incorrect") == 0) {
			calcHeadStatus = (CALCHEAD_INCORRECT);
		}
	}

	public String getStatusString() {
		return statusString;
	}

	/**
	 * distinguish between (1) the _overall_-status of a CalcHead and (2) e.g.
	 * the CALCHEAD_CORRECT for model items as defined above HERE (1) IS MEANT !
	 * 
	 * @see <isac.util.formulae.CalcHead#setCalcHeadStatus(int)>
	 */
	public void setStatusString(String string) {//TOD0.WN041007 rename to
		// setCalcHeadStatus
		statusString = string; //TOD0.WN041007 calcHeadStatus = string;
		setCalcHeadStatus(string);//TOD0.WN041007 drop this !!!
	}

	public boolean usesTheory(String thyID) {//TODO.WN040930 new type TheoryID
		// ?
		return this.specification.getTheory().equals(thyID);
	}

	public boolean usesSubTheoryOf(String thyID) {//TODO.WN040930 new type
		// TheoryID ?
		System.err.println("TODO CalcHead.usesSubTheoryOf(" + thyID + ")");
		System.exit(999);
		return false;
	}

	public boolean usesProblem(ProblemID pid) {
		return this.specification.getProblem().equals(pid);
	}

	public boolean usesProblemClass(ProblemID pid) {
		CalcHeadCompoundID p = this.specification.getProblem();
		return ((ProblemID) this.specification.getProblem())
				.belongsToClass(pid);
	}

}