/*
 * Created on Aug 16, 2003 
 */

/**
 * Class to gather all functions which use XSLT to one class to enhance the maintainability.
 *  
 */
package isac.util.genhtml;

import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.transform.Source;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamSource;

/**
 * @@author Griesmayer Andreas
 *
 * Encoderclass to build the HTML-Code out of XML-informations using XSLT
 */
public class Encoder {
	/**
	 * This method acts as the <code>encoder</code> part of the BrowserServlet
	 * 
	 * @@param xsl_source description how to encode the data as XSL Stylesheet
	 * @@param xml_source data to encode
	 * @@param writer output
	 */
	static public void encodeInformation(Source xsl_source, Source xml_source , Writer writer){
	javax.xml.transform.TransformerFactory tFactory = javax.xml.transform.TransformerFactory.newInstance();
	javax.xml.transform.Transformer transformer;
	try {
		transformer = tFactory.newTransformer(xsl_source);
		// Perform the transformation, sending the output to the response.
		transformer.transform(xml_source, new javax.xml.transform.stream.StreamResult(writer));		
	} catch (TransformerConfigurationException e) {
		e.printStackTrace();
	} catch (TransformerException e) {
		e.printStackTrace();
	}
	}

	/**
	 * Used to transform the XML representation of the hierarchy into HTML+javascript. This method also uses 
	 * two static files which contain the header and footer of the javascript-code to create the tree-view.
	 * This method is called by the class LoadFile when <code>code.html</code> is requested.
	 *    
	 * @@param type Type of the tree to load the proper XSL-stylesheet
	 * @@param xml_source hierarchy in its XML-representation
	 * @@param writer Writer to put the result in.
	 * @@see LoadFile#loadFile
	 */
	static public void encodeHierarchy(String type, Source xml_source , Writer writer){
	javax.xml.transform.TransformerFactory tFactory = javax.xml.transform.TransformerFactory.newInstance();
	javax.xml.transform.Transformer transformer;
	Source xsl_source = loadHierarchyXSL(type);
	try {
		transformer = tFactory.newTransformer(xsl_source);
		// Perform the transformation, sending the output to the response.
		String header = toString(loadTextFile("/home/neuper/proto2/isac/src/java/isac/util/genhtml/codeheader.html"));
		String footer = toString(loadTextFile("/home/neuper/proto2/isac/src/java/isac/util/genhtml/codefooter.html"));
		writer.write(header);
		transformer.transform(xml_source, new javax.xml.transform.stream.StreamResult(writer));
		writer.write(footer);
	} catch (TransformerConfigurationException e) {
		e.printStackTrace();
	} catch (TransformerException e) {
		e.printStackTrace();
	} catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	}
    
	static private Source loadHierarchyXSL(String type){
	String xsl_file= "";
	if(type.equals("pbl"))
		xsl_file="/home/neuper/proto2/isac/src/java/isac/util/genhtml/templates/pbl_hierarchy_jmenu.xsl";
	else if(type.equals("met"))
		xsl_file="/home/neuper/proto2/isac/src/java/isac/util/genhtml/templates/met_hierarchy_jmenu.xsl";
	else
		xsl_file="/home/neuper/proto2/isac/src/java/isac/util/genhtml/templates/exp_hierarchy_jmenu.xsl";
	return new StreamSource(loadTextFile(xsl_file));
	}

	static private Source loadHContentXSL(String type){
	String xsl_file= "";
	if(type.equals("pbl"))
		xsl_file="/home/neuper/proto2/isac/xmldata/pbl/pbl_hcontent.xsl";
	else if(type.equals("met"))
		xsl_file="/home/neuper/proto2/isac/xmldata/met/met_hcontent.xsl";
	else
		xsl_file="/home/neuper/proto2/isac/xmldata/exp/exp_hcontent.xsl";
	return new StreamSource(loadTextFile(xsl_file));
	}

	private static Reader loadTextFile(String filename){
	try{
		return new InputStreamReader((new URL("file", "", filename)).openStream());
	} catch (MalformedURLException e) {
		return new StringReader(getErrorXML("Sorry, MalformedURLException while accessing file "+filename+": "+e.getMessage()));
	} catch (IOException e) {
		return new StringReader(getErrorXML("Sorry, IOException while accessing file"+filename+": "+e.getMessage()));
	}
	}

	private static String toString(Reader istream){
	StringBuffer buf = new StringBuffer(10000);
	int input;
	try {
		while( (input = istream.read()) != -1 ) {
		buf.append((char)input);
		}
	} catch (IOException e) {
		return getErrorXML("Sorry, IOException in Method toString");
	}
	return buf.toString();
	}

	/** 
	 * Encodes the hierarchy of an <code>type</code> in an simple output-format. This method is not used in
	 * regulary operation. It is used by the loadFile-class when the content of a type is requested - request of 
	 * file <code>treecontent.html</code>. (e.g. to use a  mirror-tool like wget to fetch all files of a type automatically)
	 * 
	 * @@param type The type of information to load. e.g. <code>pbl, met</code> used to fetch the fitting
	 * XSL-file
	 * @@param xml_source Hierarchy to encode 
	 * @@param writer Used to write the output to.
	 * @@see LoadFile#loadFile
	 */
	public static void encodeHContent(String type, Source xml_source , Writer writer){
		javax.xml.transform.TransformerFactory tFactory = javax.xml.transform.TransformerFactory.newInstance();
		javax.xml.transform.Transformer transformer;
		Source xsl_source = loadHContentXSL(type);
		try {
			transformer = tFactory.newTransformer(xsl_source);
			// Perform the transformation, sending the output to the response.
			String header = "<html><body>";
			String footer = "</body></html>";
			writer.write(header);
			transformer.transform(xml_source, new javax.xml.transform.stream.StreamResult(writer));
			writer.write(footer);
		} catch (TransformerConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}


	/** This method creates a XML-String which can be displayed on the Webbrowser in case of an Exception
	 * @@param string The error-message to embed within the XML
	 * @@return XML-String to display on the screen
	 */
	private static String getErrorXML(String string) {
		return "<ERROR><DESCRIPTION> <h1> Error in file Encoder.java </h1>"+string+"</DESCRIPTION></ERROR>";
	}

	/**
	 * @param type
	 * @param xmlSource
	 * @param out
	 */
	public static void encodeHierarchy(String type, Source xmlSource, FileWriter out) {
		// TODO Auto-generated method stub
		
	}

}
