(* use"xmlsrc/datatypes.sml";
   use"datatypes.sml";
   *)

val i = indentation;

(** general types: lists,  **)

fun id2xml j ids =
    let fun id2x _ [] = ""
	  | id2x j (s::ss) = (indt j) ^ "<STRING> " ^ s ^ " </STRING>\n" ^ 
			     (id2x j ss)
    in (indt j) ^ "<STRINGLIST>\n" ^ 
       (id2x (j + indentation) ids) ^ 
       (indt j) ^ "</STRINGLIST>\n" end;
(* writeln(id2xml 8 ["linear","univariate","equation"]);
        <STRINGLIST>
          <STRING>linear</STRING>
          <STRING>univariate</STRING>
          <STRING>equation</STRING>
        </STRINGLIST>*)

fun ints2xml j ids =
    let fun int2x _ [] = ""
	  | int2x j (s::ss) = (indt j) ^"<INT> "^ string_of_int s ^" </INT>\n"^
			     (int2x j ss)
    in (indt j) ^ "<INTLIST>\n" ^ 
       (int2x (j + i) ids) ^ 
       (indt j) ^ "</INTLIST>\n" end;
(* writeln(ints2xml 3 [1,2,3]);
   *)


(** isac datatypes **)

fun pos_2xml j pos_ =
    (indt j) ^ "<POS> " ^  pos_2str pos_ ^ " </POS>\n";

(*.due to specialties of isac/util/parser/XMLParseDigest.java
   pos' requires different tags.*)
fun pos'2xml j (tag, (pos, pos_)) =
    indt     (j) ^ "<" ^ tag ^ ">\n" ^ 
    ints2xml (j+i) pos ^ 
    pos_2xml (j+i) pos_ ^ 
    indt     (j) ^ "</" ^ tag ^ ">\n";
(* writeln(pos'2xml 3 ("POSITION", ([1,2,3], Pbl)));
   *)

fun rule2xml j Erule =
    (indt j ^ "<RULE> empty_rule </RULE>\n"):xml
  | rule2xml j (Thm (thmID, thm)) =
    indt j ^ "<RULE>\n" ^
    indt (j+i) ^ "<THEOREMID>" ^ thmID ^ "</THEOREMID>\n" ^
    term2xml j ((#prop o rep_thm) thm) ^ "\n" ^
    indt j ^ "</RULE>\n"
  | rule2xml j (Calc (op_, _)) =
    indt j ^ "<RULE>\n" ^
    indt (j+i) ^"<CALCULATE> (" ^ op_ ^ ", eval_fun) </CALCULATE>\n" ^
    indt j ^ "</RULE>\n"
  | rule2xml j (Rls_ rls) =
    indt j ^ "<RULE>\n" ^
    indt (j+i) ^"<RULESET> " ^ ((#id o rep_rls) rls) ^ " </RULESET>\n" ^
    indt j ^ "</RULE>\n";
fun rules2xml j [] = ("":xml)
  | rules2xml j (r::rs) = rule2xml j r ^ rules2xml j rs;

fun filterpbl str =
  let fun filt [] = []
        | filt ((s, (t1, t2)) :: ps) = 
	  if str = s then (t1 $ t2) :: filt ps else filt ps
  in filt end;

(*FIXME.WN040831 model2xml <--?--> pattern2xml*)
(*WN.25.8.03: pattern2xml different output with TextIO | writeln !!!
  the version below is for TextIO: terms2xml makes \n !*)
fun pattern2xml j p where_ =
    (case filterpbl "#Given" p of
	[] =>  (indt j) ^ "<GIVEN>  </GIVEN>\n"
      | gis => (indt j) ^ "<GIVEN>\n" ^ terms2xml j gis ^
	       (indt j) ^ "</GIVEN>\n")
    ^ 
    (case where_ of
	 [] =>  (indt j) ^ "<WHERE>  </WHERE>\n"
       | whs => (indt j) ^ "<WHERE>\n" ^ terms2xml j whs ^
		(indt j) ^ "</WHERE>\n")
    ^ 
    (case filterpbl "#Find" p of
	 [] =>  (indt j) ^ "<FIND>  </FIND>\n"
       | fis => (indt j) ^ "<FIND>\n" ^ terms2xml j fis ^
		(indt j) ^ "</FIND>\n")
    ^ 
    (case filterpbl "#Relate" p of
	 [] =>  (indt j) ^ "<RELATE>  </RELATE>\n"
       | res => (indt j) ^ "<RELATE>\n" ^ terms2xml j res ^
		(indt j) ^ "</RELATE>\n");
(*
writeln(pattern2xml 3 ((#ppc o get_pbt)
			 ["squareroot","univariate","equation","test"]) []);
  *)

(*see itm_2item*)
fun itm_2xml j (Cor (dts,_))= 
    (indt j ^"<ITEM status=\"correct\">\n"^    
    term2xml (j) (comp_dts' dts)^"\n"^    
    indt j ^"</ITEM>\n"):xml
  | itm_2xml j (Syn c) =
    indt j ^"<ITEM status=\"syntaxerror\">\n"^    
    indt (j) ^c^    
    indt j ^"</ITEM>\n"
  | itm_2xml j (Typ c) =
    indt j ^"<ITEM status=\"typeerror\">\n"^    
    indt (j) ^c^    
    indt j ^"</ITEM>\n"
  | itm_2xml j (Inc (dts,_)) = 
    indt j ^"<ITEM status=\"incomplete\">\n"^    
    term2xml (j) (comp_dts' dts)^"\n"^    
    indt j ^"</ITEM>\n"
  | itm_2xml j (Sup dts) = 
    indt j ^"<ITEM status=\"superfluous\">\n"^    
    term2xml (j) (comp_dts' dts)^"\n"^    
    indt j ^"</ITEM>\n"
  | itm_2xml j (Mis (d,pid)) =
    indt j ^"<ITEM status=\"superfluous\">\n"^    
    term2xml (j) (d $ pid)^"\n"^    
    indt j ^"</ITEM>\n";

(*see terms2xml' fpr \n*)
fun itms2xml _ [] = ""
  | itms2xml j [(_,_,_,_,itm_)] = itm_2xml j itm_
  | itms2xml j (((_,_,_,_,itm_):itm)::itms) = 
    itm_2xml j itm_ ^ itms2xml j itms;

fun precond2xml j (true, term) =
    (indt j ^"<ITEM status=\"correct\">\n"^    
    term2xml (j) term^"\n"^    
    indt j ^"</ITEM>\n"):xml
  | precond2xml j (false, term) =
    indt j ^"<ITEM status=\"false\">\n"^    
    term2xml (j+i) term^"\n"^    
    indt j ^"</ITEM>\n";

fun preconds2xml _ [] = ("":xml)
  | preconds2xml j (p::ps) = precond2xml j p ^ preconds2xml j ps;

(*FIXME.WN040831 model2xml <--?--> pattern2xml*)
fun model2xml j (itms:itm list) where_ =
    let fun eq4 str (_,_,_,field,_) = str = field
    in  (indt j ^"<MODEL>\n"^
	(case filter (eq4 "#Given") itms of
	     [] =>  (indt (j+i)) ^ "<GIVEN>  </GIVEN>\n"
	   | gis => (indt (j+i)) ^ "<GIVEN>\n" ^ itms2xml (j+2*i) gis ^
		    (indt (j+i)) ^ "</GIVEN>\n")
	^ 
	(case where_ of
	     [] =>  (indt (j+i)) ^ "<WHERE>  </WHERE>\n"
	   | whs => (indt (j+i)) ^ "<WHERE>\n" ^ preconds2xml (j+2*i) whs ^
		    (indt (j+i)) ^ "</WHERE>\n")
	^ 
	(case filter (eq4 "#Find") itms of
	     [] =>  (indt (j+i)) ^ "<FIND>  </FIND>\n"
	   | fis => (indt (j+i)) ^ "<FIND>\n" ^ itms2xml (j+2*i) fis ^
		    (indt (j+i)) ^ "</FIND>\n")
	^ 
	(case filter (eq4 "#Relate") itms of
	     [] =>  (indt (j+i)) ^ "<RELATE>  </RELATE>\n"
	   | res => (indt (j+i)) ^ "<RELATE>\n" ^ itms2xml (j+2*i) res ^
		    (indt (j+i)) ^ "</RELATE>\n")^
	    indt j ^"</MODEL>\n"):xml
    end;
(* writeln(model2xml 3 itms []);
   *)

fun spec2xml j ((dI,pI,mI):spec) =
    (indt j ^"<SPECIFICATION>\n"^ 
    indt (j+i) ^"<THEORYID> "^ dI ^" </THEORYID>\n"^
    indt (j+i) ^"<PROBLEMID>\n"^
    id2xml (j+2*i) pI ^
    indt (j+i) ^"</PROBLEMID>\n"^
    indt (j+i) ^"<METHODID>\n"^
    id2xml (j+2*i) mI ^
    indt (j+i) ^"</METHODID>\n"^
    indt j ^"</SPECIFICATION>\n"):xml;

fun modspec2xml j ((b, p_, head, gfr, pre, spec): ocalhd) =
    (indt j ^"<CALCHEAD status = "^
     quote (if b then "correct" else "incorrect")^">\n"^
     indt (j+i) ^"<HEAD>\n"^
     term2xml (j+i) head^"\n"^
     indt (j+i) ^"</HEAD>\n"^ 
     model2xml (j+i) gfr pre ^
     indt (j+i) ^"<BELONGSTO> "^(case p_ of Pbl => "Pbl" 
					  | Met => "Met"
					  | _ => "Und")^" </BELONGSTO>\n"^ 
     spec2xml (j+i) spec ^
     indt j ^"</CALCHEAD>\n"):xml;
(* writeln (modspec2xml 2 e_ocalhd);
   *)
fun pos'calchead2xml j (p:pos', (b, p_, head, gfr, pre, spec): ocalhd) =
    (indt j ^"<CALCHEAD status = "^
     quote (if b then "correct" else "incorrect")^">\n"^
     pos'2xml (j+i) ("POSITION", p) ^ 
     indt (j+i) ^"<HEAD>\n"^
     term2xml (j+i) head^"\n"^
     indt (j+i) ^"</HEAD>\n"^ 
     model2xml (j+i) gfr pre ^
     indt (j+i) ^"<BELONGSTO> "^(case p_ of Pbl => "Pbl" 
					  | Met => "Met"
					  | _ => "Und")^" </BELONGSTO>\n"^ 
     spec2xml (j+i) spec ^
     indt j ^"</CALCHEAD>\n"):xml;

fun sub2xml j (id, value) = 
    (indt j ^"<PAIR>\n"^ 
     indt j ^"  <VARIABLE>\n"^ 
     term2xml (j+i) id ^ "\n" ^
     indt j ^"  </VARIABLE>\n" ^
     indt j ^"  <VALUE>\n"^ 
     term2xml (j+i) value ^ "\n" ^
     indt j ^"  </VALUE>\n" ^
     indt j ^"</PAIR>\n"):xml;

fun subs2xml j (subs:subs) = 
    (indt j ^"<SUBSTITUTION>\n"^
     foldl op^ ("", map (sub2xml (j+i)) 
			(subs2subst (assoc_thy "Isac.thy") subs)) ^
     indt j ^"</SUBSTITUTION>\n"):xml;
(* val subs = [(str2term "bdv", str2term "x")];
   writeln(subs2xml 0 subs);
   *)



fun thmstr2xml j str = ((((term2xml j) o str2term) str)^"\n"):xml;

fun thm'2xml j ((ID, form):thm') =
    (indt j ^"<THEOREM>\n"^
    indt (j+i) ^"<ID> "^ID^" </ID>\n"^
    indt (j+i) ^"<FORMULA>\n"^
    thmstr2xml (j+i) form^
    indt (j+i) ^"</FORMULA>\n"^
    indt j ^"</THEOREM>\n"):xml;

fun tac2xml j (Subproblem (dI, pI)) =
    (indt j ^"<SUBPROBLEMTACTIC name=\"Subproblem\">\n"^    
    indt (j+i) ^"<THEORY> "^ dI ^" </THEORY>\n"^
    indt (j+i) ^"<PROBLEM>\n"^
    id2xml (j+2*i) pI^
    indt (j+i) ^"</PROBLEM>\n"^    
    indt j ^"</SUBPROBLEMTACTIC>\n"):xml
  | tac2xml j (Model_Problem pI) =
    (indt j ^"<STRINGLISTTACTIC name=\"Model_Problem\">\n"^    
    id2xml (j+i) pI^    
    indt j ^"</STRINGLISTTACTIC>\n"):xml
  | tac2xml j (Refine_Tacitly pI) =
    (indt j ^"<STRINGLISTTACTIC name=\"Refine_Tacitly\">\n"^    
    id2xml (j+i) pI^    
    indt j ^"</STRINGLISTTACTIC>\n"):xml

  | tac2xml j (Add_Given ct) =
    (indt j ^"<SIMPLETACTIC name=\"Add_Given\">\n"^    
    cterm2xml (j+i) ct^    
    indt j ^"</SIMPLETACTIC>\n"):xml
  | tac2xml j (Add_Find ct) =
    (indt j ^"<SIMPLETACTIC name=\"Add_Find\">\n"^    
    cterm2xml (j+i) ct^    
    indt j ^"</SIMPLETACTIC>\n"):xml
  | tac2xml j (Add_Relation ct) =
    (indt j ^"<SIMPLETACTIC name=\"Add_Relation\">\n"^    
    cterm2xml (j+i) ct^    
    indt j ^"</SIMPLETACTIC>\n"):xml

  | tac2xml j (Specify_Theory ct) =
    (indt j ^"<SIMPLETACTIC name=\"Specify_Theory\">\n"^    
    cterm2xml (j+i) ct^    
    indt j ^"</SIMPLETACTIC>\n"):xml
  | tac2xml j (Specify_Problem ct) =
    (indt j ^"<STRINGLISTTACTIC name=\"Specify_Problem\">\n"^    
    id2xml (j+i) ct^    
    indt j ^"</STRINGLISTTACTIC>\n"):xml
  | tac2xml j (Specify_Method ct) =
    (indt j ^"<STRINGLISTTACTIC name=\"Specify_Method\">\n"^    
    id2xml (j+i) ct^    
    indt j ^"</STRINGLISTTACTIC>\n"):xml
  | tac2xml j (Apply_Method mI) =
    (indt j ^"<STRINGLISTTACTIC name=\"Apply_Method\">\n"^    
    id2xml (j+i) mI^    
    indt j ^"</STRINGLISTTACTIC>\n"):xml

  | tac2xml j (Rewrite thm') =
    (indt j ^"<REWRITETACTIC name=\"Rewrite\">\n"^    
    thm'2xml (j+i) thm'^    
    indt j ^"</REWRITETACTIC>\n"):xml
(* writeln (tac2xml 2 (Rewrite ("all_left",
				"~ ?b =!= 0 ==> (?a = ?b) = (?a - ?b = 0)")));
   *)
  | tac2xml j (Rewrite_Inst (subs, thm')) =
    (indt j ^"<REWRITEINSTTACTIC name=\"Rewrite_Inst\">\n"^    
    subs2xml (j+i) subs^
    thm'2xml (j+i) thm'^    
    indt j ^"</REWRITEINSTTACTIC>\n"):xml
(* writeln (tac2xml 2 (Rewrite_Inst 
			   (["(bdv,x)"], 
			    ("all_left",
			     "~ ?b =!= 0 ==> (?a = ?b) = (?a - ?b = 0)"))));
   *)
  | tac2xml j (Rewrite_Set rls') =
    (indt j ^"<REWRITESETTACTIC name=\"Rewrite_Set\">\n"^    
    indt (j+i) ^"<RULESET> "^ rls' ^" </RULESET>\n"^    
    indt j ^"</REWRITESETTACTIC>\n"):xml
  | tac2xml j (Rewrite_Set_Inst (subs, rls')) =
    (indt j ^"<REWRITESETINSTTACTIC name=\"Rewrite_Set_Inst\">\n"^    
    indt (j+i) ^"<RULESET> "^ rls' ^" </RULESET>\n"^
    subs2xml (j+i) subs^    
    indt j ^"</REWRITESETINSTTACTIC>\n"):xml

  | tac2xml j (Or_to_List) =
    (indt j ^"<STRINGLISTTACTIC name=\"Or_to_List\"> \
	     \</STRINGLISTTACTIC>\n"):xml
  | tac2xml j (Check_elementwise ct) =
    (indt j ^"<SIMPLETACTIC name=\"Check_elementwise\">\n"^    
    term2xml (j+i) 
	     (str2term ct
	      (*FIXME.WN.9.03: redesign cterm' for xml-output*)) ^ "\n"^    
    indt j ^"</SIMPLETACTIC>\n"):xml
  | tac2xml j (Check_Postcond pI) =
    (indt j ^"<STRINGLISTTACTIC name=\"Check_Postcond\">\n"^    
    id2xml (j+i) pI^    
    indt j ^"</STRINGLISTTACTIC>\n"):xml

  | tac2xml j tac = raise error ("tac2xml: not impl. for "^tac2str tac);

fun tacs2xml j [] = "":xml
  | tacs2xml j (t::ts) = tac2xml j t ^ tacs2xml j ts; 


fun posformhead2xml j (p:pos', Form f) =
    indt j ^"<CALCFORMULA>\n"^
    pos'2xml (j+i) ("POSITION", p) ^ 
    indt (j+i) ^"<FORMULA>\n"^
    term2xml (j+i) f^"\n"^
    indt (j+i) ^"</FORMULA>\n"^
    indt j ^"</CALCFORMULA>\n"
  | posformhead2xml j (p, ModSpec c) =
    pos'calchead2xml (j) (p, c);

fun posformheads2xml j [] = ("":xml)
  | posformheads2xml j (r::rs) = posformhead2xml j r ^ posformheads2xml j rs;
